% this guide generator version takes into account the deformation on the
% boundary in order to avoid seeing the edges effect. for this the plankg
% is generated in a larger image and then cropped
%
% Version 0.4, september 2017
% Author lara.raad@gmail.com
%
% level sets method
% Creates random guide planks using level sets of a smooth function with
% up to two Gaussian bumps.
%
% Version 0.2, May 2017
% Author jlezama@fing.edu.uy

function [s1,g1,g2,s2,s3,s4] = guide_generator(params)

% params = [H W sigma_blob sigma_disp max_disp blob_vert_pos...
% blob_hor_pos num_int blob_mult dilat_vert dilat_hor surf_decay...
% vert_disp_mult]

% check number of input parameters
errormsg = 'there should be 20 specified parameters';
assert(length(params)==20,errormsg);

% get parameters
r = params(1); % ratio H/W
W1 = params(2); % width
H1 = r*W1; % height
% extension of H and W to have an output of size HxW
px = params(4)*1.1; % horizontal extension factor
py = params(5)*1.1; % vertical extension factor
H = H1+2*ceil(H1*py);
W = W1+2*ceil(W1*px);

sigma_displacement = round(params(3)*min(H1,W1));
max_displacement_x = params(4)*W1;
max_displacement_y = params(5)*H1;
blob_vertical_position =  params(6:7);
blob_horizontal_position = params(8:9);
num_intervals = params(10);
blob_multiplier = params(11:12);
surface_decay = params(13);

rho1 = params(14);
excentricity1 = params(15);
sig1 = params(16)*min(H1,W1);
mu11 = round(H1*blob_vertical_position(1)) + ...
    ceil(H1*py)*(blob_vertical_position(1)>=0);
mu12 = round(W1*blob_horizontal_position(1)) + ...
    ceil(W1*px)*(blob_horizontal_position(1)>=0);

rho2 = params(17);
excentricity2 = params(18);
sig2 = params(19)*min(H1,W1);
mu21 = round(H1*blob_vertical_position(2)) + ...
    ceil(H1*py)*(blob_vertical_position(2)>=0);
mu22 = round(W1*blob_horizontal_position(2)) + ...
    ceil(W1*px)*(blob_horizontal_position(2)>=0);

rnd_wgt = params(20);

%gaussian parameters
%blob 1 covariance matrix
sig1_x = sig1;
sig1_y = sig1/excentricity1;

sig1_11 = sig1_x^2*cosd(rho1)^2+sig1_y^2*sind(rho1)^2;
sig1_22 = sig1_y^2*cosd(rho1)^2+sig1_x^2*sind(rho1)^2;
sig1_12 = (sig1_x^2-sig1_y^2)*cosd(rho1)*sind(rho1);
Sigma1 = [sig1_11 sig1_12;sig1_12 sig1_22];

%blob 2  covariance matrix
sig2_x = sig2;
sig2_y = sig2/excentricity2;

sig2_11 = sig2_x^2*cosd(rho2)^2+sig2_y^2*sind(rho2)^2;
sig2_22 = sig2_y^2*cosd(rho2)^2+sig2_x^2*sind(rho2)^2;
sig2_12 = (sig2_x^2-sig2_y^2)*cosd(rho2)*sind(rho2);
Sigma2 = [sig2_11 sig2_12;sig2_12 sig2_22];

% initalize surface
[X, ~] = meshgrid(1:W,1:H);
initial_surface = (1-normalize(abs(X-W/2).^surface_decay));

initial_surface(isnan(initial_surface))=0;

s1 = initial_surface;
% cut borders
s1 = cut_borders(s1,H1,W1,px,py);

% add two gaussian bumps
disp('generating gaussian blob 1')
g1 = gaussian_blob(H,W,mu11,mu12,Sigma1);

disp('generating gaussian blob 2')
g2 = gaussian_blob(H,W,mu21,mu22,Sigma2);

mysurface = initial_surface + g1*blob_multiplier(1) + g2*blob_multiplier(2);
mysurface = normalize(mysurface);

% add random gaussian blobs
disp('generating random gaussian blobs')
a = random_gaussian_blobs(H,W,100)*rnd_wgt;
mysurface = normalize(mysurface + a);

s2 = mysurface;
% cut borders
s2 = cut_borders(s2,H1,W1,px,py);

% add some random displacement
disp('generating random displacement')
[vx, vy] = random_displacement(H, W, sigma_displacement, max_displacement_x, max_displacement_y, 0, 0 ,0);
mysurface = apply_random_displacement(mysurface, vx, vy);

mysurface = normalize(mysurface);
mysurface(isnan(mysurface))=0;
s3 = mysurface;
% cut borders
s3 = cut_borders(s3,H1,W1,px,py);

% resize image
mysurface = imresize(s3,6);
mysurface = normalize(mysurface);
[H, W] = size(mysurface);

%% compute Level Lines
intervals = 0:1/num_intervals:1;

intervals = normalize(1-nonlinear(intervals, -.5)); % stretch intervals to have larger gaps in cathedrals
intervals = sort(normalize(intervals + randn(size(intervals))/300));
L = length(intervals);

% colorize
red = zeros(H,W);
green = zeros(H,W);
blue = zeros(H,W);

% light = [255, 204, 102]/255. % [0.8513 0.7047 0.4893]; %[229 203 162]/255;
light = [252, 248, 241]/255; % [157, 157, 157]/255; % [209, 161, 98]/255. % [0.8513 0.7047 0.4893]; %[229 203 162]/255;
% dark = ([230, 153, 0])/255;
dark = [211, 209, 196]/255;%  [142 128 117]/255; %([181, 132, 68])/255;

disp('painting intervals')

bands = zeros(H,W);
for i=1:L-1
    z = find(mysurface>=intervals(i) & mysurface<=intervals(i+1));
    
    % bands(z) = nonlinear(1-normalize(mysurface(z)), 3);
    bands(z) = 1-normalize(mysurface(z));
    % bands(z) = normalize(sigmoid(1-normalize(mysurface(z)), .85, 100));
    % bands(z) = normalize(sigmoid2(1-(mysurface(z))));
    
    dark_tmp = dark;% + randn(1,3)/R;
    light_tmp = light;% + randn(1,3)/R;
    
    red(z) = dark_tmp(1) * bands(z) + light_tmp(1) * (1-bands(z));
    green(z) = dark_tmp(2) * bands(z) + light_tmp(2) * (1-bands(z));
    blue(z) = dark_tmp(3) * bands(z) + light_tmp(3) * (1-bands(z));
    
    %figure; imshow(cat(3,red,green,blue),[])
    %pause(.1)
    
end


% % % bands = zeros(H,W);
% % % for i=1:L-1
% % %     z = find(mysurface>=intervals(i) & mysurface<=intervals(i+1));
% % %     
% % %     % bands(z) = nonlinear(1-normalize(mysurface(z)), 3);
% % %     % bands(z) = 1-normalize(mysurface(z));
% % %     % bands(z) = normalize(sigmoid(1-normalize(mysurface(z)), .85, 100));
% % %     bands(z) = normalize(sigmoid2(1-normalize(mysurface(z))));
% % %     
% % %     dark_tmp = dark;% + randn(1,3)/R;
% % %     light_tmp = light;% + randn(1,3)/R;
% % %     
% % %     red(z) = dark_tmp(1) * bands(z) + light_tmp(1) * (1-bands(z));
% % %     green(z) = dark_tmp(2) * bands(z) + light_tmp(2) * (1-bands(z));
% % %     blue(z) = dark_tmp(3) * bands(z) + light_tmp(3) * (1-bands(z));
% % %     
% % %     %figure; imshow(cat(3,red,green,blue),[])
% % %     %pause(.1)
% % %     
% % % end

new_colored = cat(3,red,green,blue);
s4 = new_colored;                      
% cut borders
% s4 = cut_borders(s4,H1,W1,px,py);